d_setregid='undef'
d_setreuid='undef'
d_setrgid='undef'
d_setruid='undef'
d_setegid='undef'
d_seteuid='undef'
alignbytes=8
dolist_cflags='optimize=""'
tdolist_cflags='optimize=""'
regexec_cflags='optimize=""'
tregexec_cflags='optimize=""'
eval_cflags='optimize=""'
teval_cflags='optimize=""'
toke_cflags='optimize=""'
ttoke_cflags='optimize=""'
ccflags="$ccflags -D_NO_PROTO"
cppstdin='/lib/cpp -D_AIX -D_IBMR2 -U__STR__'
cppminus=''
