#!/bin/sh
# config.sh
# This file was produced by running the Configure script.
d_eunice='undef'
define='define'
eunicefix=':'
loclist='
cat
cp
echo
expr
grep
mkdir
mv
rm
sed
sort
tr
uniq
'
expr='/usr/bin/expr'
sed='/usr/bin/sed'
echo='echo'
cat='/usr/bin/cat'
rm='/usr/bin/rm'
mv='/usr/bin/mv'
cp='/usr/bin/cp'
tail=''
tr='/usr/bin/tr'
mkdir='/usr/bin/mkdir'
sort='/usr/bin/sort'
uniq='/usr/bin/uniq'
grep='/usr/bin/grep'
trylist='
Mcc
bison
cpp
csh
egrep
line
nroff
perl
test
uname
yacc
'
test='test'
inews=''
egrep='/usr/bin/egrep'
more=''
pg=''
Mcc='Mcc'
vi=''
mailx=''
mail=''
cpp='/usr/lib/cpp'
perl='/usr/bin/perl'
emacs=''
ls=''
rmail=''
sendmail=''
shar=''
smail=''
tbl=''
troff=''
nroff='/usr/bin/nroff'
uname='/usr/bin/uname'
uuname=''
line='/usr/bin/line'
chgrp=''
chmod=''
lint=''
sleep=''
pr=''
tar=''
ln=''
lpr=''
lp=''
touch=''
make=''
date=''
csh='/usr/bin/csh'
bash=''
ksh=''
lex=''
flex=''
bison='bison'
Log='$Log'
Header='$Header'
Id='$Id'
lastuname='SunOS beach 4.1.2 2 sun4m'
alignbytes='8'
bin='/local/bin'
installbin='/local/bin'
byteorder='4321'
contains='grep'
cppstdin='/usr/lib/cpp'
cppminus=''
d_bcmp='define'
d_bcopy='define'
d_safebcpy='define'
d_bzero='define'
d_castneg='define'
castflags='0'
d_charsprf='define'
d_chsize='undef'
d_crypt='define'
cryptlib=''
d_csh='define'
d_dosuid='undef'
d_dup2='define'
d_fchmod='define'
d_fchown='define'
d_fcntl='define'
d_flexfnam='define'
d_flock='define'
d_getgrps='define'
d_gethent='undef'
d_getpgrp='define'
d_getpgrp2='undef'
d_getprior='define'
d_htonl='define'
d_index='undef'
d_isascii='define'
d_killpg='define'
d_lstat='define'
d_memcmp='define'
d_memcpy='define'
d_safemcpy='undef'
d_memmove='undef'
d_memset='define'
d_mkdir='define'
d_msg='define'
d_msgctl='define'
d_msgget='define'
d_msgrcv='define'
d_msgsnd='define'
d_ndbm='define'
d_odbm='define'
d_open3='define'
d_readdir='define'
d_rename='define'
d_rewindir='define'
d_rmdir='define'
d_seekdir='define'
d_select='define'
d_sem='define'
d_semctl='define'
d_semget='define'
d_semop='define'
d_setegid='define'
d_seteuid='define'
d_setpgrp='define'
d_setpgrp2='undef'
d_setprior='define'
d_setregid='define'
d_setresgid='undef'
d_setreuid='define'
d_setresuid='undef'
d_setrgid='define'
d_setruid='define'
d_shm='define'
d_shmat='define'
d_voidshmat='undef'
d_shmctl='define'
d_shmdt='define'
d_shmget='define'
d_socket='define'
d_sockpair='define'
d_oldsock='undef'
socketlib=''
d_statblks='define'
d_stdstdio='define'
d_strctcpy='define'
d_strerror='define'
d_symlink='define'
d_syscall='define'
d_telldir='define'
d_truncate='define'
d_vfork='define'
d_voidsig='define'
d_tosignal='int'
d_volatile='undef'
d_vprintf='define'
d_charvspr='define'
d_wait4='define'
d_waitpid='define'
gidtype='gid_t'
groupstype='int'
i_fcntl='undef'
i_gdbm='undef'
i_grp='define'
i_niin='define'
i_sysin='undef'
i_pwd='define'
d_pwquota='undef'
d_pwage='define'
d_pwchange='undef'
d_pwclass='undef'
d_pwexpire='undef'
d_pwcomment='define'
i_sys_file='define'
i_sysioctl='define'
i_time='undef'
i_sys_time='define'
i_sys_select='undef'
d_systimekernel='undef'
i_utime='define'
i_varargs='define'
i_vfork='define'
intsize='4'
libc='/usr/lib/libc.so.1.7.1'
nm_opts=''
libndir=''
i_my_dir='undef'
i_ndir='undef'
i_sys_ndir='undef'
i_dirent='define'
i_sys_dir='undef'
d_dirnamlen='undef'
ndirc=''
ndiro=''
mallocsrc='malloc.c'
mallocobj='malloc.o'
d_mymalloc='define'
mallocptrtype='char'
mansrc='/local/man/man1'
manext='1'
models='none'
split=''
small=''
medium=''
large=''
huge=''
optimize='-O'
ccflags=' -I/usr/include/sun -I/usr/ucbinclude'
cppflags=' -I/usr/include/sun -I/usr/ucbinclude'
ldflags=''
cc='cc'
nativegcc=''
libs='-lnsl -ldbm -lm -lposix'
n='-n'
c=''
package='perl'
randbits='31'
scriptdir='/local/bin'
installscr='/local/bin'
sig_name='ZERO HUP INT QUIT ILL TRAP ABRT EMT FPE KILL BUS SEGV SYS PIPE ALRM TERM URG STOP TSTP CONT CLD TTIN TTOU IO XCPU XFSZ VTALRM PROF WINCH LOST USR1 USR2'
spitshell='cat'
shsharp='true'
sharpbang='#!'
startsh='#!/bin/sh'
stdchar='unsigned char'
uidtype='uid_t'
usrinclude='/usr/include'
inclPath=''
void=''
voidhave='7'
voidwant='7'
w_localtim='1'
w_s_timevl='1'
w_s_tm='1'
yacc='/usr/bin/yacc'
lib=''
privlib='/local/lib/perl'
installprivlib='/local/lib/perl'
PATCHLEVEL=35
CONFIG=true
