# Hints file (perl 4.019) for Kubota Pacific's Titan 3000 Series Machines.
# Created by: JT McDuffie (jt@kpc.com)  26 DEC 1991
bin='/usr/local/bin'
installbin='/usr/local/bin'
alignbytes="8"
byteorder="4321"
cppstdin='/lib/cpp'
cppminus=''
castflags='0'
gid_type='ushort'
groupstype='unsigned short'
intsize='4'
libc='/lib/libc.a'
nm_opts='-eh'
mallocptrtype='void'
mansrc='/usr/man/man1'
installmansrc='/usr/man/man1'
manext='1'
models='none'
optimize='-O'
ccflags="$ccflags -I/usr/include/net -DDEBUGGING"
cppflags="$cppflags -I/usr/include/net -DDEBUGGING"
cc='cc'
libs='-lnsl -ldbm -lPW -lmalloc -lm'
libswanted='net socket nsl nm ndir ndbm dbm PW malloc m x posix '
scriptdir='/usr/local/bin'
installscr='/usr/local/bin'
stdchar='unsigned char'
uidtype='ushort'
usrinclude='/usr/include'
voidhave='7'
w_localtim='1'
w_s_timevl='1'
w_s_tm='1'
privlib='/usr/local/lib/perl'
installprivlib='/usr/local/lib/perl'
inclwanted='/usr/include /usr/include/net '
libpth=' /usr/lib /usr/local/lib /lib'
eoPATH='/bin /usr/bin /usr/ucb /usr/local /usr/local/bin /usr/lbin /etc /usr/lib /lib /usr/local/lib '
pth=' . /bin /usr/bin /usr/ucb /usr/local/bin /usr/X11/bin /usr/lbin /etc /usr/lib /lib /usr/local/lib '
